//***************************************************************************************************************************************************
//** 																																			   **				
//**	Pinbelegung																																   **
//**	----------------------------------------------------																					   **
//**	ATMEGA168          	RFM12		HH10D		Platine																						   **
//**	----------------------------------------------------																					   **
//**	SCK               	SCK																													   **
//**	MISO              	SDO																													   **
//**	MOSI              	SDI																													   **
//**	PB2                	nSEL																												   **
//**	PD2 (INT0)        	nIRQ																												   **
//**	PB0               	FSK																													   **
//**	PD3 (INT1)        	FFIT																												   **
//**	PB1               	VDI	/ nINT (Wird nicht bentigt)																											   **
//**	----------------------------------------------------																					   **
//** 	PD5               							Free																					       **
//** 	PD6               							Free																						   **
//**	----------------------------------------------------																					   **
//**	SDA								SDA																										   **
//**	SCL								SCL																										   **
//**	PD4 (T0)						FOUT																									   **
//** 																																			   **
//**    Modus a: Temperatur, Feuchtigkeit, Taupunkt 																							   **
//***************************************************************************************************************************************************


#include <avr/io.h>
#include <stdlib.h>
#include <string.h>
#include <avr/eeprom.h>
#include <avr/wdt.h>

#include "spi.h"
#include "i2c.h"
#include "rfm12.h"
#include "uart.h"
#include "hh10d.h"
#include "ds1621.h"
#include "taupunkt.h"
#include "power.h"
#include "timer.h"

#define F_CPU 16000000UL

#define AktorenCounterInit	(sizeof(AktorenListe) / sizeof(uint8_t))-1
#define AktorenTimeoutInit	1
#define AktorZustand	PIND & (1<<PIND4)

uint8_t adr EEMEM;
uint8_t typ EEMEM;

char Temp_Array[5];																			// Hilfsarray fr Umwandlungen in String
char Text_Array[20];																		// Sendearray fr das RFM12
char FunkPacket[20];

char Temp[2];
char Modus[2];
char ZielAdresse[2];

uint8_t MessungFertig = False, FAN_Status = False;
uint16_t LaufZeitCounter=0;
float StableHumi = 0,StableHumiOld = 0,ActTemp = 0, RecHumi=0;

uint8_t AktorenListe[3];

uint8_t AktorenTimeout = 0, AktorenAnfrage = False, AktorenCounter = 0;




int main(void)
{
//	eeprom_write_byte (&adr, 3);
//	eeprom_write_byte (&typ, 2);	

	DDRB  = 0xff;																		// PORTC als Ausgang
	DDRC  = 0xff;																		// PORTC als Ausgang
	DDRD  = 0xff;																		// PORTC als Ausgang

	AktorenCounter = AktorenCounterInit;
	AktorenListe[1] = 5;
	AktorenListe[2] = 4;

	Adresse[0] = eeprom_read_byte (&adr);	

	if(eeprom_read_byte (&typ) == 1)TypMode = Master;

	if(eeprom_read_byte (&typ) == 2)TypMode = Sensor;

	if(eeprom_read_byte (&typ) == 3)TypMode = Aktor;

	uart_init();																			// UART initialisieren	
	spi_master_init();																		// SPI initialisieren	
	rfm12_init();																			// RFM12 initialisieren	
	timer_init();																			// init Timer
	sei();

	// ***************************** Startbetrieb ***************************************************************************************************

	if (TypMode == Master)																	// Wenn Modul = Master
	{
		uart_send_text("Firmware v2.0.2");
		uart_send_byte('\r');
		uart_send_byte('\r');

		Modus[0] = 'd';																		// Modus setzen, fr Aktorabfrage
	}

	if (TypMode == Sensor)																	// Wenn Modul = Sensor
	{
		SENSOREN_ON;																		// Sensoren einschalten
		i2c_init();																			// IC initialisieren
		ds1621_init();																		// init DS1621
		hh10d_init();																		// init HH10D
		timer_reset();
	}																		

	if (TypMode == Aktor)																	// Wenn Modul = Aktor
	{
		DDRD |= (1 << PD4);																	// PD4 (FAN) als Ausgang

		Modus[0] = 'd';																		// Modus setzen
		ZielAdresse[0] = 1;																	// Zieladresse setzen (hier Master)
		strcpy( Text_Array, Adresse);														// Modul Adresse in Sendearray kopieren
		strcat( Text_Array, ZielAdresse );													// Zieladresse in Sendearray kopieren
		strcat( Text_Array, Modus );														// Modus in Sendearray kopieren
							
		if(AktorZustand)Temp[0] = '1';
		else Temp[0] = '0';
		
		strcat( Text_Array,Temp);
							
		rfm12_send_text(Text_Array);														// Sendearray ber RFM12 versenden	
	}

	// **********************************************************************************************************************************************

	
	if (TypMode == Sensor)
	{		
		//standby();		
	}
	

	while(True)																	
	{
		// ***************************** Normalbetrieb **********************************************************************************************
										
		if(rfm12_fifo_voll == True)															// Lst aus wenn FIFO von RFM12 voll ist
		{
			rfm12_get_text(FunkPacket);
		}
	
		if (TypMode == Master)																// Wenn Modul = Master
		{
			if(AktorenCounter > 0)
			{
				if(AktorenAnfrage == False)
				{
					
					ZielAdresse[0] = AktorenListe[AktorenCounter];							// Zieladresse setzen (hier Master)
					strcpy( Text_Array, Adresse);											// Modul Adresse in Sendearray kopieren
					strcat( Text_Array, ZielAdresse );										// Zieladresse in Sendearray kopieren
					strcat( Text_Array, Modus );											// Modus in Sendearray kopieren		
					rfm12_send_text(Text_Array);											// Sendearray ber RFM12 versenden
					AktorenAnfrage = True;												
				}
			}


			if(PacketEmpfangen == True)
			{		
				switch(FunkPacket[3])														// Modus ausgewerten
				{
					case 'a':						
						Modus[0] = 'a';														// Modus setzen
						ZielAdresse[0] = FunkPacket[1];										// Zieladresse setzen
				
						strcpy( Text_Array, Adresse);										// Modul Adresse in Sendearray kopieren
						strcat( Text_Array, ZielAdresse );									// Zieladresse in Sendearray kopieren
						strcat( Text_Array, Modus );										// Modus in Sendearray kopieren
						rfm12_send_text(Text_Array);										// Sendearray ber RFM12 versenden	


						uart_send_text("Sender.........: ");
						itoa(FunkPacket[1],Temp_Array,10);				
						uart_send_text(Temp_Array);									
						uart_send_byte('\r');
		
						uart_send_text("Modus..........: ");
						uart_send_byte(FunkPacket[3]);
						uart_send_byte('\r');

						uart_send_text("Feuchtigkeit...: ");
						uart_send_byte(FunkPacket[4]);
						uart_send_byte(FunkPacket[5]);
						uart_send_byte(FunkPacket[6]);
						uart_send_byte(FunkPacket[7]);
						uart_send_byte('\r');

						uart_send_text("Temperatur.....: ");
						uart_send_byte(FunkPacket[8]);
						uart_send_byte(FunkPacket[9]);
						uart_send_byte(FunkPacket[10]);
						uart_send_byte(FunkPacket[11]);
						uart_send_byte('\r');

						uart_send_text("Taupunkt.......: ");
						uart_send_byte(FunkPacket[12]);
						uart_send_byte(FunkPacket[13]);
						uart_send_byte(FunkPacket[14]);
						uart_send_byte(FunkPacket[15]);
						uart_send_byte('\r');

						uart_send_text("Nummer.........: ");
						uart_send_byte(FunkPacket[16]);
						uart_send_byte(FunkPacket[17]);
						uart_send_byte(FunkPacket[18]);
						uart_send_byte(FunkPacket[19]);
						uart_send_byte('\r');
						
						Temp_Array[0] = FunkPacket[4];
						Temp_Array[1] = FunkPacket[5];
						Temp_Array[2] = FunkPacket[6];
						Temp_Array[3] = FunkPacket[7];
						RecHumi = (float)atof(Temp_Array);						
					break;

					case 'b': 										
						FAN_Status = True;						
						uart_send_text("Aktor Adresse ");
						itoa(FunkPacket[1],Temp_Array,10);				
						uart_send_text(Temp_Array);									
						uart_send_text(" wurde eingeschaltet");
						uart_send_byte('\r');

						if(AktorenCounter > 0)
						{
							AktorenCounter--;
							AktorenAnfrage = False;
							AktorenTimeout = AktorenTimeoutInit;
						}
						else
						{
							AktorenAnfrage = False;
							AktorenTimeout = AktorenTimeoutInit;							
						}
					break;

					case 'c': 	
						FAN_Status = False;															
						uart_send_text("Aktor Adresse ");
						itoa(FunkPacket[1],Temp_Array,10);				
						uart_send_text(Temp_Array);									
						uart_send_text(" wurde ausgeschaltet");
						uart_send_byte('\r');

						if(AktorenCounter > 0)
						{
							AktorenCounter--;
							AktorenAnfrage = False;
							AktorenTimeout = AktorenTimeoutInit;
						}
						else
						{
							AktorenAnfrage = False;
							AktorenTimeout = AktorenTimeoutInit;							
						}

					break;

					case 'd': 	
						if(FunkPacket[4] == '0')
						{
							FAN_Status = False;
						}
						else 
						{
							FAN_Status = True; 
						}

						uart_send_text("Aktor Adresse ");
						itoa(FunkPacket[1],Temp_Array,10);				
						uart_send_text(Temp_Array);									
						
						if(FAN_Status == True)uart_send_text(" ist an");
						if(FAN_Status == False)uart_send_text(" ist aus");

						uart_send_byte('\r');
						
						
						if(AktorenCounter > 0)
						{
							AktorenCounter--;
							AktorenAnfrage = False;
							AktorenTimeout = AktorenTimeoutInit;
						}
						else
						{
							AktorenAnfrage = False;
							AktorenTimeout = AktorenTimeoutInit;							
						}
					break;
				}
				uart_send_byte('\r');				

				if(RecHumi > 65 && FAN_Status == False)
				{
					Modus[0] = 'b';	
					AktorenCounter = AktorenCounterInit;				
				}
				
				if(RecHumi > 0 && RecHumi < 60 && FAN_Status == True)
				{
					Modus[0] = 'c';
					AktorenCounter = AktorenCounterInit;				
				}

				PacketEmpfangen = False;													// Empfangs-Merker zurcksetzen
			}
		}
		
		if (TypMode == Sensor)																// Wenn Modul = Sensor
		{
			if(PacketEmpfangen == True)
			{
				PacketEmpfangen = False;													// Empfangs-Merker zurcksetzen
				standby();				
			}

			if(MessungFertig == True)
			{
				Modus[0] = 'a';																// Modus setzen
				ZielAdresse[0] = 1;															// Zieladresse setzen (hier Master)
				ActTemp = ds1621_get_temp();												// Aktuelle Temperatur einlesen
				
				ActTemp = 25.0;
				StableHumi = 45.0;

				strcpy( Text_Array, Adresse);												// Modul Adresse in Sendearray kopieren
				strcat( Text_Array, ZielAdresse );											// Zieladresse in Sendearray kopieren
				strcat( Text_Array, Modus );												// Modus in Sendearray kopieren		
				dtostrf( StableHumi, 4, 1,Temp_Array);										// Luftfeuchtigkeit messen und in String umwandeln
				strcat( Text_Array, Temp_Array );											// Luftfeuchtigkeit in Sendearray kopieren			
				dtostrf( ActTemp, 4, 1, Temp_Array);										// Temperatur messen und in String umwandeln
				strcat( Text_Array, Temp_Array );											// Temperatur in Sendearray kopieren
				
				taupunkt = get_taupunkt(StableHumi,ActTemp);
				taupunkt = 8.0;

				dtostrf(taupunkt , 4, 1, Temp_Array);				// Taupunkt berechnen und in String umwandeln
				strcat( Text_Array, Temp_Array );											// Taupunkt in Sendearray kopieren
				LaufZeitCounter++;															// Laufende Nummer hochzhlen
				dtostrf(LaufZeitCounter, 4, 0, Temp_Array);									// Laufende Nummer umwandeln
				strcat( Text_Array, Temp_Array );											// Taupunkt in Sendearray kopieren		
				rfm12_send_text(Text_Array);												// Sendearray ber RFM12 versenden												
				MessungFertig = False;
			}
		}
		
		if (TypMode == Aktor)																// Wenn Modul = Aktor
		{
			if(PacketEmpfangen == True)
			{
				switch(FunkPacket[3])														// Modus ausgewerten
				{
					case 'b': 																// Lfter-Modus															
						FAN_ON;																// Lfter einschalten					
						Modus[0] = 'b';														// Modus setzen
						ZielAdresse[0] = 1;													// Zieladresse setzen (hier Master)				
						strcpy( Text_Array, Adresse);										// Modul Adresse in Sendearray kopieren
						strcat( Text_Array, ZielAdresse );									// Zieladresse in Sendearray kopieren
						strcat( Text_Array, Modus );										// Modus in Sendearray kopieren
						rfm12_send_text(Text_Array);										// Sendearray ber RFM12 versenden	
					break;
						
					case 'c': 																// Lfter-Modus
						FAN_OFF;															// Lfter ausschalten
						Modus[0] = 'c';														// Modus setzen
						ZielAdresse[0] = 1;													// Zieladresse setzen (hier Master)
						strcpy( Text_Array, Adresse);										// Modul Adresse in Sendearray kopieren
						strcat( Text_Array, ZielAdresse );									// Zieladresse in Sendearray kopieren
						strcat( Text_Array, Modus );										// Modus in Sendearray kopieren
						rfm12_send_text(Text_Array);										// Sendearray ber RFM12 versenden	
					break;				

					case 'd': 																// Lfter-Modus
						Modus[0] = 'd';														// Modus setzen
						ZielAdresse[0] = 1;													// Zieladresse setzen (hier Master)
						strcpy( Text_Array, Adresse);										// Modul Adresse in Sendearray kopieren
						strcat( Text_Array, ZielAdresse );									// Zieladresse in Sendearray kopieren
						strcat( Text_Array, Modus );										// Modus in Sendearray kopieren
							
						if(AktorZustand)Temp[0] = '1';
						else Temp[0] = '0';
						strcat( Text_Array,Temp );
							
						rfm12_send_text(Text_Array);										// Sendearray ber RFM12 versenden	
					break;					
				}
				PacketEmpfangen = False;													// Empfangs-Merker zurcksetzen
			}
		}

		// ******************************************************************************************************************************************



		// ***************************** Intervallbetrieb 1 Sekunde (8ms x 125) *********************************************************************

		if (IntervallSekunde == True)
		{
			if (TypMode == Master)															// Wenn Modul = Master
			{
				if(AktorenAnfrage == True)
				{
					if(AktorenTimeout > 0)
					{
						AktorenTimeout--;
					}
					else
					{
						if(AktorenCounter > 0)
						{
							uart_send_text("Aktor Adresse ");
							utoa(AktorenListe[AktorenCounter], ZielAdresse, 10);
							uart_send_byte(ZielAdresse[0]);
							uart_send_text(" nicht erreichbar");
							uart_send_byte('\r');
							uart_send_byte('\r');

							AktorenCounter--;
							AktorenAnfrage = False;
							AktorenTimeout = AktorenTimeoutInit;
						}
						else
						{
							AktorenAnfrage = False;
							AktorenTimeout = AktorenTimeoutInit;							
						}
					}

				}
			}

			if (TypMode == Sensor)															// Wenn Modul = Sensor
			{	
				StableHumi = hh10d_get_humi();
				if(StableHumiOld == 0)StableHumiOld = StableHumi;							// init StableHumiOld
				StableHumi = (StableHumiOld + StableHumi) / 2;								// Mittelwert bilden
				StableHumiOld = StableHumi;													// Mittelwert sichern
				MessungFertig = True;				
			}

			if (TypMode == Aktor)															// Wenn Modul = Aktor
			{
				asm("nop");
			}
							
			IntervallSekunde = False;														// Intervall-Merker zurcksetzen
		}

		// ******************************************************************************************************************************************
	}
}
