#include "rfm12.h"

void rfm12_init(void)
{
	for(uint8_t t=0;t<30;t++)_delay_ms(10);								// Warten bis RFM12 hochgefahren
	DDRB  |=  (1<<PB0);													// PB0 (nFFS) als Ausgang 	(Wird nicht bentigt, wenn ein Pullup vorhanden ist)
	PORTB |=  (1<<PB0);													// nFFS auf high ziehen  	(Wird nicht bentigt, wenn ein Pullup vorhanden ist)
	DDRD  &= ~(1<<PIND2);												// PD2(INT0) fr den nIRQ-Pin	
	DDRD  &= ~(1<<PIND3);				 								// PD3(INT1) fr den FIFO-Interrupt
	EICRA |= (1<<ISC10) | (1<<ISC11);									// INT1 steigende Flanke
	EIMSK  |= (1<<INT1);												// INT1 aktivieren
	rfm12_write(0xC000);												// AVR CLK: 1MHz
	rfm12_write(0x80D7);												// Enable FIFO
	rfm12_write(0xC2AB);												// Data Filter: internal
	rfm12_write(0xCA81);												// Set FIFO mode
	rfm12_write(0xC4F7);												// AFC settings: autotuning: -10kHz...+7,5kHz
	rfm12_write(0x82C8);												// Empfnger einschalten
	rfm12_write(0xCA81);												// set FIFO mode
	rfm12_write(0xCA83);												// enable FIFO	
	rfm12_setfreq(RF12FREQ(433.92));									// Sende/Empfangsfrequenz auf 433,92MHz einstellen
	rfm12_setbandwidth(4, 1, 4);										// 200kHz Bandbreite, -6dB Verstrkung, DRSSI threshold: -79dBm 
	rfm12_setbaud(RFM12_BAUD);											// Baudrate einstellen
	rfm12_setpower(0, 6);												// 1mW Ausgangangsleistung, 120kHz Frequenzshift	
	rfm12_write(0x0000);												// Interrupts lschen
}

void rfm12_setbandwidth(uint8_t bandwidth, uint8_t gain, uint8_t drssi)
{
	rfm12_write(0x9400|((bandwidth&7)<<5)|((gain&3)<<3)|(drssi&7));		// Bandbreite einstellen
}

void rfm12_setfreq(uint16_t freq)
{	if (freq<96)														// 430,2400MHz
		freq=96;
	else if (freq>3903)													// 439,7575MHz
		freq=3903;
	rfm12_write(0xA000|freq);
}

void rfm12_setbaud(uint16_t baud)
{
	if (baud<664)
		baud=664;
	if (baud<5400)														// Baudrate= 344827,58621/(R+1)/(1+CS*7)
		rfm12_write(0xC680|((43104/baud)-1));							// R=(344828/8)/Baud-1
	else
		rfm12_write(0xC600|((344828UL/baud)-1));						// R=344828/Baud-1
}

void rfm12_setpower(uint8_t power, uint8_t mod)
{	
	rfm12_write(0x9800|(power&7)|((mod&15)<<4));
}

void rfm12_write_data(uint16_t wert)
{
	rfm12_write(wert);													// Sendeword bertragen
	while(PIND & (1<<PIND2));											// Warten bis Modul bereit (nIRQ = 0)

}

uint16_t rfm12_write(uint16_t wert)
{
	return spi_trans_16(wert);											// Rckgabe des empfangenen Words
}

uint8_t rfm12_read(void)
{
	uint16_t FIFO_data;							
	rfm12_write(0x0000);												// Status Register lesen (Interrupts werden gelscht)
	while(PIND & (1<<PIND2));											// Warten bis Modul bereit (nIRQ = 0)
	FIFO_data=rfm12_write(0xB000);										// FIFO-Daten einlesen (16Bit)
	return(FIFO_data&0x00FF);											// LOW-Byte der FIFO-Daten zurckgeben (Datenbyte)
}


void rfm12_send_text(char *s)
{
	cli();																// Interrupts sperren
	LED1_ON;															// LED Senden ein
	PacketLen = strlen(s);												// Packetlnge einlesen
	rfm12_write(0x8208);												// RX off	
	rfm12_write_data(0x8238);											// Sender einschalten
	rfm12_write_data(0xB8AA);											// Prambel senden			
	rfm12_write_data(0xB8AA);											// Prambel senden						
	rfm12_write_data(0xB8AA);											// Prambel senden						
	rfm12_write_data(0xB82D);											// Syncronmuster senden			
	rfm12_write_data(0xB8D4);											// Syncronmuster senden				

	rfm12_write_data(0xB800 | PacketLen);								// Nutzerbyte senden

	uint8_t BitCounter=0;
	for(uint8_t c=0;c<8;c++)
	{
		if((PacketLen & (1<<c))>0)BitCounter++;
	}
	
	for(uint8_t i=0;i<PacketLen;i++)
	{
		for(uint8_t c=0;c<8;c++)
		{
			if((*s & (1<<c))>0)BitCounter++;
		}

		rfm12_write_data(0xB800 | *s);									// Nutzerbyte senden
		s++;
	}

	rfm12_write_data(0xB800 | BitCounter);
	
	rfm12_write_data(0xB8AA);											// Dummybyte
	rfm12_write_data(0xB8AA);											// Dummybyte		
	rfm12_write_data(0x8208);											// Sender ausschalten		
	rfm12_write(0x82C8);												// Empfnger einschalten	
	LED1_OFF;															// LED Senden aus
	sei();																// Interrupts wieder freigeben
}


void rfm12_get_text(char *s)
{	
	if(i==0)
	{
		LED2_ON;														// LED Empfangen ein
		rfm12_get_status;												// Interrupts lschen (Wichtig!)
		PacketLen = rfm12_read();				
		s[0] = PacketLen;				
		BitCounterRec=0;
		cli();
		for(uint8_t c=0;c<8;c++)
		{
			if((PacketLen & (1<<c))>0)BitCounterRec++;
		}
		sei();
	}

	if(i<PacketLen)
	{
		uint8_t EmpfangsWert = rfm12_read();
		cli();
		for(uint8_t c=0;c<8;c++)
		{
			if((EmpfangsWert & (1<<c))>0)BitCounterRec++;
		}
		sei();
		s[i+1] = EmpfangsWert;													
		i++;															// Zhler hochzhlen
	}

	if(i==PacketLen)
	{
		uint8_t BitCounterSoll = rfm12_read();
		if(BitCounterSoll == BitCounterRec && s[2] == Adresse[0])		// Wenn Prfsumme passt und wenn Empfngeradresse passt
		{
			PacketEmpfangen = 1;								
		}
		else 
		{
			PacketEmpfangen = 0;
		}

		i=0;												
		rfm12_fifo_reset;												// FIFO wieder freigeben (Wichtig!)
		rfm12_fifo_voll=0;
		LED2_OFF;														// LED Empfangen aus
		timer_reset();
	}
}


ISR(INT1_vect)															// FIFO-Interrupt (PIN: FFIT)
{
	rfm12_fifo_voll=1;													// ISR Variable setzen	
}
